@@ -64,77 +64,9 @@ static void gnome_cups_request_connection_destroy (GnomeCupsConnection *conn);
 static gboolean idle_stop_unused_threads (gpointer unused);
 static gboolean idle_close_unused_connections (gpointer unused);
 
-gboolean _gnome_cups_debug = FALSE;
-
-static const char *
-get_error_string (ipp_status_t error)
-{  
-	static const char *status_oks[] =     /* "OK" status codes */
-                {
-			N_("Success"),
-			N_("Success, some attributes were ignored"),
-			N_("Success, some attributes conflicted"),
-			N_("Success, ignored subscriptions"),
-			N_("Success, ignored notifications"),
-			N_("Success, too many events"),
-			N_("Success, cancel subscription"),
-                };
-	static const char *status_400s[] =        /* Client errors */
-		{
-			N_("Bad request"),
-			N_("Forbidden"),
-			N_("The client has not been authenticated"),
-			N_("You are not authorized to perform this operation"),
-			N_("This operation cannot be completed"),
-			N_("Timeout"),
-			N_("The requested file was not found"),
-			N_("The requested resource no longer exists"),
-			N_("The request was too large"),
-			N_("The request was too long"),
-			N_("The document format is not supported"),
-			N_("The value or attributes in this request are not supported"),
-			N_("The URI scheme is not supported"),
-			N_("The requested character set is not supported"),
-			N_("There were conflicting attributes in the request"),
-			N_("Compression is not supported"),
-			N_("There was a compression error"),
-			N_("There was an error in the format of the document"),
-			N_("There was an error accessing the document"),
-			N_("Some attributes could not be set"),
-			N_("All subscriptions were ignored"),
-			N_("Too many subscriptions"),
-			N_("All notifications were ignored."),
-			N_("A print support file was not found.")
-		};
-	
-	static const char *status_500s[] =        /* Server errors */
-                {
-			N_("Internal server error"),
-			N_("Operation not supported"),
-			N_("Service unavailable"),
-			N_("Version not supported"),
-			N_("Device error"),
-			N_("Temporary error"),
-			N_("The printer is not accepting jobs"),
-			N_("The printer is busy"),
-			N_("The job has been canceled"),
-			N_("Multiple-document jobs are not supported"),
-			N_("The printer is deactivated"),
-		};                                                                             
-	if (error >= IPP_OK && error <= IPP_OK_BUT_CANCEL_SUBSCRIPTION) {
-		return _(status_oks[error]);
-	} else if (error == IPP_REDIRECTION_OTHER_SITE) {
-		return _("Redirected to another site");
-	} else if (error >= IPP_BAD_REQUEST && error <= IPP_PRINT_SUPPORT_FILE_NOT_FOUND) {
-		return _(status_400s[error - IPP_BAD_REQUEST]);
-	} else if (error >= IPP_INTERNAL_ERROR && error <= IPP_PRINTER_IS_DEACTIVATED) {
-		return _(status_500s[error - IPP_INTERNAL_ERROR]);
-	}                                                                       
-	return _("Unknown error");
-}
-
 /* Should be per thread with push/pop/user_data etc. (clearly) */
 static GnomeCupsAuthFunction global_auth = NULL;
+static gboolean _gnome_cups_debug = FALSE;
 
 static const char *
 cups_password_cb (const char *prompt)
@@ -339,7 +271,7 @@ do_signal_complete (GnomeCupsRequest *request)
 }
 
 static void
-dump_request (ipp_t *req)
+dump_request (ipp_t const *req)
 {
 	unsigned i;
 	ipp_attribute_t *attr;
@@ -380,8 +312,6 @@ static void
 request_thread_main (GnomeCupsRequest *request,
 		     G_GNUC_UNUSED gpointer unused)
 {
-	ipp_status_t status;
-
 	g_return_if_fail (request != NULL);
 
 	if (request->canceled) {
@@ -405,7 +335,9 @@ request_thread_main (GnomeCupsRequest *request,
 	if (!request->connection->http)
 		request->connection->http = httpConnectEncrypt (request->connection->server, ippPort(), cupsEncryption());
 
-	if (request->request) {
+	if (request->request) {		/* IPP request */
+		ipp_status_t status;
+
 		request->response = cupsDoRequest (request->connection->http,
 			request->request, request->path);
 
@@ -416,12 +348,28 @@ request_thread_main (GnomeCupsRequest *request,
 		status = cupsLastError ();
 		if (request->response == NULL)
 			status = IPP_INTERNAL_ERROR;
-	} else if (request->output_fd >= 0) {
-		status = cupsGetFd(request->connection->http, request->path, request->output_fd);
+
+		if (status > IPP_OK_CONFLICT) {
+			g_warning ("IPP request failed with status %d", status);
+			if (request->error != NULL)
+				*(request->error) = g_error_new (GNOME_CUPS_ERROR, 
+								 status, ippErrorString (status));
+		} else if (request->response && _gnome_cups_debug) {
+			g_print ("response = \n");
+			dump_request (request->response);
+		}
+	} else if (request->output_fd >= 0) {	/* File request */
+		http_status_t status = cupsGetFd (request->connection->http,
+			request->path, request->output_fd);
 		request->response = NULL;
+		if (HTTP_OK != status && request->error != NULL)
+			*(request->error) = g_error_new (GNOME_CUPS_ERROR, 
+							 status, httpStatus (status));
 	} else {
-		status = IPP_INTERNAL_ERROR;
-		g_warning ("Neither request nor output_fd set\n");
+		g_warning ("Neither request nor output_fd set");
+		if (request->error != NULL)
+			*(request->error) = g_error_new (GNOME_CUPS_ERROR, 0,
+				"CORRUPT request that lacked both an ipp-request and an output_fd");
 	}
 
 	g_atomic_int_dec_and_test (&request->connection->refcount);
@@ -430,20 +378,7 @@ request_thread_main (GnomeCupsRequest *request,
 		g_print ("<<<<----- unlocking %p\n", request->connection);
 	}
 
-	if (status > IPP_OK_CONFLICT) {
-		g_warning ("failed request with status %d", status);
-		if (request->error != NULL)
-			*(request->error) = g_error_new (GNOME_CUPS_ERROR, 
-							 status,
-							 get_error_string (status));
-	} else if (request->response && _gnome_cups_debug) {
-		g_print ("response = \n");
-		dump_request (request->response);
-	}
-
 	do_signal_complete (request);
-	
-	return;
 }
 
 guint
